package com.winterchen.controller;

import com.winterchen.dao.ProductInfoMapper;
import com.winterchen.model.ProductInfo;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(value = "/redis")
@Slf4j
public class RedisController {

    @Autowired
    StringRedisTemplate template;

    static final String KEY_USER_INFO__NAME = "com_demo_user_info_007_%s";


    @PostMapping("/add")
    public String setUserNameById(Integer uid, String userName) {
        template.opsForValue().set(String.format(KEY_USER_INFO__NAME, uid), userName);
        return "ok";
    }

    @GetMapping("/get")
    public String getUserNameById(Integer uid) {
        String userName = "未知用户";
        try {
            userName = template.opsForValue().get(String.format(KEY_USER_INFO__NAME, uid));
            if (StringUtils.isEmpty(userName)) {
                return "error";
            }
        } catch (Exception e) {
            System.out.println(e.toString());
        }
        return userName;
    }

    @Autowired
    private RedissonClient redissonClient;
    //商品秒杀核心业务逻辑的处理-redisson的分布式锁

    @Autowired
    private ProductInfoMapper productInfoMapper;

    @GetMapping("/rush")
    public String decrementProductStore(Long productId, Integer productQuantity) {
//        String key = "dec_store_lock_" + productId;
//        RLock lock = redissonClient.getLock(key);
//        boolean locked = lock.isLocked();
//        if (!locked) {
//            try {
//                //加锁 操作很类似Java的ReentrantLock机制
//                lock.lock();
//                ProductInfo productInfo = productInfoMapper.selectByPrimaryKey(productId);
//                //如果库存为空
//                if (productInfo.getProductStock() < productQuantity) {
//                    return "卖完了!";
//                }
//                //简单减库存操作 没有重新写其他接口了
//                productInfo.setProductStock(productInfo.getProductStock() - productQuantity);
//                productInfoMapper.updateByPrimaryKey(productInfo);
//            } catch (Exception e) {
//                System.out.println(e.getMessage());
//            } finally {
//                //解锁
//                lock.unlock();
//            }
//            return "下单成功";
//        } else {
//            return "请重新抢购!";
//        }

        String key = "dec_store_lock_" + productId;
        RLock lock = redissonClient.getLock(key);
        boolean locked = lock.isLocked();
        try {
            //加锁 操作很类似Java的ReentrantLock机制
            lock.lock();
            ProductInfo productInfo = productInfoMapper.selectByPrimaryKey(productId);
            //如果库存为空
            if (productInfo.getProductStock() < productQuantity) {
                return "卖完了!";
            }
            //简单减库存操作 没有重新写其他接口了
            productInfo.setProductStock(productInfo.getProductStock() - productQuantity);
            productInfoMapper.updateByPrimaryKey(productInfo);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            return "请重新抢购!";
        } finally {
            //解锁
            lock.unlock();
        }
        return "下单成功";

    }


}
